@@ -42,7 +42,6 @@ extern GMainLoop *event_loop;
 static GSource *exit_timer = NULL;
 
 static gboolean exit_cb(gpointer user_data) {
-    error("btsdp running but no requests received.");
     g_main_loop_quit(event_loop);
     return FALSE;
 }
@@ -62,20 +61,23 @@ static void remove_exit_timer(void) {
     }
 }
 
+static void reset_exit_timer(void) {
+    remove_exit_timer();
+    set_exit_timer();
+}
+
 static DBusHandlerResult btname_req_handler(DBusConnection     *connection,
                                             DBusMessage        *message,
                                             void               *user_data) {
     DBusMessage *reply;
 
-    remove_exit_timer();
-
-    g_main_loop_quit(event_loop);
+    reset_exit_timer();
 
     if (dbus_message_is_method_call(message,
                                     BTNAME_REQUEST_IF,
                                     BTNAME_REQ_GET)) {
-
         char name[MAXNAMELEN];
+        char *ptr = name;
 
         debug("Method call: get()");
 
@@ -84,7 +86,7 @@ static DBusHandlerResult btname_req_handler(DBusConnection     *connection,
         else {
             reply = new_dbus_method_return(message);
             append_dbus_args(reply,
-                             DBUS_TYPE_STRING, name,
+                             DBUS_TYPE_STRING, &ptr,
                              DBUS_TYPE_INVALID);
         }
 
@@ -101,9 +103,9 @@ static DBusHandlerResult btname_req_handler(DBusConnection     *connection,
 
         debug("Method call: set()");
 
-        if (!get_dbus_args(message,
-                           DBUS_TYPE_STRING, &new_name,
-                           DBUS_TYPE_INVALID)) {
+        if (!dbus_message_get_args(message, NULL,
+                                   DBUS_TYPE_STRING, &new_name,
+                                   DBUS_TYPE_INVALID)) {
             send_invalid_args(connection, message);
             return DBUS_HANDLER_RESULT_HANDLED;
         }
@@ -115,8 +117,6 @@ static DBusHandlerResult btname_req_handler(DBusConnection     *connection,
             reply = new_dbus_method_return(message);
         }
 
-        dbus_free(new_name);
-
         if (!send_and_unref(connection, reply))
             error("Sending D-BUS reply failed!");
 
@@ -158,12 +158,15 @@ void destroy_dbus_handlers(DBusConnection *connection) {
 gboolean send_name_changed_signal(DBusConnection *connection, const char *new_name) {
     DBusMessage *signal;
 
+    if (!connection)
+        return FALSE;
+
     signal = new_dbus_signal(BTNAME_SIGNAL_PATH,
                              BTNAME_SIGNAL_IF,
                              BTNAME_SIG_CHANGED,
                              NULL);
     append_dbus_args(signal,
-                     DBUS_TYPE_STRING, new_name,
+                     DBUS_TYPE_STRING, &new_name,
                      DBUS_TYPE_INVALID);
 
     if (!send_and_unref(connection, signal)) {
