@@ -52,12 +52,12 @@ int main(int argc, char *argv[]) {
 
     report("BTName %s started.", VERSION);
 
-    if (!setup_dbus_connection(BTNAME_SERVICE, init_dbus_handlers))
-        die("D-BUS connection setup failed!");
-
     c = getopt (argc, argv, "s:gr");
     if (c == -1) {
         /* Assume that we were activated by D-BUS */
+        if (!setup_dbus_connection(BTNAME_SERVICE, init_dbus_handlers))
+            die("D-BUS connection setup failed!");
+
         event_loop = g_main_loop_new(NULL, FALSE);
 
         g_timeout_add(TIMEOUT, (GSourceFunc)dbus_timeout, event_loop);
@@ -69,7 +69,10 @@ int main(int argc, char *argv[]) {
 
         switch (c) {
             case 'g':
-                get_name(name, sizeof(name));
+                if (!get_name(name, sizeof(name))) {
+                    error("Getting name failed!\n");
+                    exit(EXIT_FAILURE);
+                }
                 printf("%s\n", name);
                 break;
 
@@ -78,8 +81,7 @@ int main(int argc, char *argv[]) {
                     error("Setting name failed!");
                     exit(EXIT_FAILURE);
                 }
-                else
-                    send_name_changed_signal(get_dbus_connection(), optarg);
+                send_name_changed_signal(get_dbus_connection(), optarg);
                 break;
 
             case 'r':
@@ -91,8 +93,7 @@ int main(int argc, char *argv[]) {
                     error("Setting name failed!");
                     exit(EXIT_FAILURE);
                 }
-                else
-                    send_name_changed_signal(get_dbus_connection(), name);
+                send_name_changed_signal(get_dbus_connection(), name);
                 break;
 
             default:
