@@ -100,7 +100,6 @@ static DBusHandlerResult btpair_req_handler(DBusConnection     *connection,
     DBusMessageIter iter;
     GError *err = NULL;
     PairingContext *ctx = NULL;
-    dbus_bool_t encrypt = FALSE;
 
     remove_exit_timer();
 
@@ -119,44 +118,53 @@ static DBusHandlerResult btpair_req_handler(DBusConnection     *connection,
 
         debug("Method call: pair_bda()");
 
-        if (!dbus_message_iter_init(message, &iter) ||
-            !get_dbus_iter_args(&iter,
-                                DBUS_TYPE_STRING, &bda,
-                                DBUS_TYPE_UINT32, &clock,
-                                DBUS_TYPE_INVALID)) {
+        if (!dbus_message_iter_init(message, &iter)) {
                 send_invalid_args(connection, message);
                 g_main_loop_quit(mainloop);
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
 
-        if (dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_BOOLEAN)
-            get_dbus_iter_args(&iter,
-                               DBUS_TYPE_BOOLEAN, &encrypt,
-                               DBUS_TYPE_INVALID);
+        dbus_message_iter_get_basic(&iter, &bda);
 
-        ctx = pair_bda(bda, clock, encrypt, (btpair_cb)btpair_dbus_cb, message, &err); 
+        if (dbus_message_iter_next(&iter))
+            dbus_message_iter_get_basic(&iter, &clock);
+
+        ctx = pair_bda(bda, clock, (btpair_cb)btpair_dbus_cb, message, &err); 
     }
-
-    if (dbus_message_is_method_call(message,
-                                    BTPAIR_INTERFACE,
-                                    BTPAIR_REQ_PAIR_HANDLE)) {
+    else if (dbus_message_is_method_call(message,
+                                         BTPAIR_INTERFACE,
+                                         BTPAIR_REQ_PAIR_HANDLE)) {
         int handle;
 
-        if (!dbus_message_iter_init(message, &iter) || 
-            !get_dbus_iter_args(&iter,
-                                DBUS_TYPE_UINT32, &handle,
-                                DBUS_TYPE_INVALID)) {
+        if (!dbus_message_iter_init(message, &iter)) {
                 send_invalid_args(connection, message);
                 g_main_loop_quit(mainloop);
                 return DBUS_HANDLER_RESULT_HANDLED;
         }
 
-        if (dbus_message_iter_get_arg_type(&iter) == DBUS_TYPE_BOOLEAN)
-            get_dbus_iter_args(&iter,
-                               DBUS_TYPE_BOOLEAN, &encrypt,
-                               DBUS_TYPE_INVALID);
+        dbus_message_iter_get_basic(&iter, &handle);
 
-        ctx = pair_handle(handle, encrypt, (btpair_cb)btpair_dbus_cb, message, &err); 
+        ctx = pair_handle(handle, (btpair_cb)btpair_dbus_cb, message, &err); 
+    }
+    else if (dbus_message_is_method_call(message,
+                                         BTPAIR_INTERFACE,
+                                         BTPAIR_REQ_REMOVE_PAIRING)) {
+        char *bda = NULL;
+
+        if (!dbus_message_get_args(message, NULL,
+                                   DBUS_TYPE_STRING, &bda,
+                                   DBUS_TYPE_INVALID))
+                send_invalid_args(connection, message);
+        else {
+            remove_link_key(bda);
+
+            reply = dbus_message_new_method_return(message);
+            if (!send_and_unref(connection, reply))
+                error("Sending D-BUS reply failed");
+        }
+
+        g_main_loop_quit(mainloop);
+        return DBUS_HANDLER_RESULT_HANDLED;
     }
 
     if (err) {
