@@ -30,6 +30,7 @@
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/file.h>
 
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/hci.h>
@@ -60,24 +61,14 @@ struct pairing_context {
     guint       io_id;
     gboolean    user_handle;
     gboolean    connected;
-    gboolean    encrypted;
     State       state;
     bdaddr_t    bdaddr;
-    gboolean    encrypt;
     btpair_cb   func;
     gpointer    user_data;
 };
 
 #define CTX_SOCK(ctx) (g_io_channel_unix_get_fd((ctx)->io))
 
-#ifndef OCF_CC_CANCEL
-#define OCF_CC_CANCEL 0x0008
-#endif
-
-#ifdef BUGGY_BT_HW
-# define AUTH_WAIT 500
-#endif
-
 #define HCI_DEV_ID      0
 #define CC_TIMEOUT      30000
 #define DC_TIMEOUT      100
@@ -115,6 +106,24 @@ static int event_socket(int *events) {
     return sock;
 }
 
+static void hci_delete_link_key(const char *bda) {
+    int sock;
+    bdaddr_t ba;
+
+    sock = hci_open_dev(HCI_DEV_ID);
+    if (sock < 0) {
+        error("hci_open_dev(%d): %s", HCI_DEV_ID, strerror(errno));
+        return;
+    }
+
+    str2ba(bda, &ba);
+
+    if (hci_delete_stored_link_key(sock, &ba, 0, 100) < 0)
+        error("hci_delete_stored_link_key failed: %s", strerror(errno));
+
+    close(sock);
+}
+
 static void get_bda_handle(bdaddr_t *ba, int *handle) {
     struct hci_conn_info_req *cr;
     int dd;
@@ -140,7 +149,7 @@ static void get_bda_handle(bdaddr_t *ba, int *handle) {
     *handle = cr->conn_info->handle;
 }       
 
-gboolean btpair_send_cmd(struct pairing_context *ctx, uint16_t ogf, uint16_t ocf,
+static gboolean btpair_send_cmd(struct pairing_context *ctx, uint16_t ogf, uint16_t ocf,
                     uint8_t plen, void *param) {
     if (hci_send_cmd(CTX_SOCK(ctx), ogf, ocf, plen, param) < 0) {
         error("hci_send_cmd: %s", g_strerror(errno));
@@ -155,13 +164,16 @@ gboolean btpair_send_cmd(struct pairing_context *ctx, uint16_t ogf, uint16_t ocf
 
 static gboolean cancel_connect(struct pairing_context *ctx) {
    status_bdaddr_rp rp; 
+   create_conn_cancel_cp cp;
    struct hci_request rq;
 
+   bacpy(&cp.bdaddr, &ctx->bdaddr);
+
    memset(&rq, 0, sizeof(rq));
    rq.ogf    = OGF_LINK_CTL;
-   rq.ocf    = OCF_CC_CANCEL;
-   rq.cparam = &ctx->bdaddr;
-   rq.clen   = sizeof(ctx->bdaddr);
+   rq.ocf    = OCF_CREATE_CONN_CANCEL;
+   rq.cparam = &cp;
+   rq.clen   = CREATE_CONN_CANCEL_CP_SIZE;
    rq.rparam = &rp;
    rq.rlen   = STATUS_BDADDR_RP_SIZE;
 
@@ -256,33 +268,6 @@ static gboolean request_auth(struct pairing_context *ctx) {
     return TRUE;
 }
 
-static gboolean set_encrypt(struct pairing_context *ctx) {
-    set_conn_encrypt_cp cp;
-
-    cp.handle = ctx->handle;
-    cp.encrypt = 0x01;
-
-    if (!btpair_send_cmd(ctx, OGF_LINK_CTL, OCF_SET_CONN_ENCRYPT,
-                         SET_CONN_ENCRYPT_CP_SIZE, &cp))
-        return FALSE;
-
-    return TRUE;
-}
-
-#ifdef BUGGY_BT_HW
-static gboolean auth_wait_cb(struct pairing_context *ctx) {
-    if (!request_auth(ctx)) {
-        GError *err;
-        g_set_error(&err, BT_PAIR_ERROR,
-                    BT_PAIR_ERROR_AUTH_FAILED,
-                    "Sending HCI_Request_Authentication failed!");
-        pair_finalize(ctx, err);
-    }
-
-    return FALSE;
-}
-#endif
-
 static int handle_conn_complete(void *data, struct pairing_context *ctx, GError **err) {
     evt_conn_complete *ev = data;
 
@@ -305,20 +290,12 @@ static int handle_conn_complete(void *data, struct pairing_context *ctx, GError
 
     debug("Connected. Handle = %u", btohs(ev->handle));
 
-#ifdef BUGGY_BT_HW
-    debug("Waiting %u milliseconds before HCI_Authentication_Requested",
-            AUTH_WAIT);
-    (void) g_timeout_add(AUTH_WAIT,
-                         (GSourceFunc)auth_wait_cb,
-                         ctx);
-#else
     if (!request_auth(ctx)) {
         g_set_error(err, BT_PAIR_ERROR,
                     BT_PAIR_ERROR_AUTH_FAILED,
                     "Sending HCI_Request_Authentication failed!");
         return 1;
     }
-#endif
 
     return 0;
 }
@@ -352,32 +329,6 @@ static int handle_disconn_complete(void *data, struct pairing_context *ctx, GErr
     return 1;
 }
 
-static int handle_encrypt_change(void *data, struct pairing_context *ctx, GError **err) {
-    evt_encrypt_change *ev = data;
-
-    if (ev->handle != ctx->handle)
-        return 0;
-
-    if (ev->status != 0) {
-        error("Encryption change failed with 0x%02X", ev->status);
-        return 1;
-    }
-
-    if (ev->encrypt == 0x01) {
-        debug("Encryption enabled");
-        ctx->encrypted = TRUE;
-    }
-    else {
-        /* Should not happen */
-        debug("Encryption disabled");
-    }
-
-    if (ctx->state == STATE_AUTHENTICATED)
-        return 1;
-
-    return 0;
-}
-
 static int handle_auth_complete(void *data, struct pairing_context *ctx, GError **err) {
     evt_auth_complete *ev = data;
 
@@ -395,16 +346,6 @@ static int handle_auth_complete(void *data, struct pairing_context *ctx, GError
 
     ctx->state = STATE_AUTHENTICATED;
 
-    if (ctx->encrypt && !ctx->encrypted) {
-        debug("Requesting encryption");
-        if (!set_encrypt(ctx)) {
-            /* Don't fail since HCI_Request_Authentication is the important part */
-            error("Sending HCI_Set_Connection_Encryption command failed!");
-            return 1;
-        }
-        return 0;
-    }
-
     return 1;
 }
 
@@ -421,8 +362,6 @@ static int process_data(char *data, int data_len, struct pairing_context *ctx, G
                 return handle_conn_complete(body, ctx, err);
             case EVT_AUTH_COMPLETE:
                 return handle_auth_complete(body, ctx, err);
-            case EVT_ENCRYPT_CHANGE:
-                return handle_encrypt_change(body, ctx, err);
             case EVT_DISCONN_COMPLETE :
                 return handle_disconn_complete(body, ctx, err);
             case EVT_CMD_STATUS:
@@ -500,7 +439,6 @@ static gboolean hci_setup(struct pairing_context *ctx) {
     int events[] = { EVT_CONN_COMPLETE,
                      EVT_DISCONN_COMPLETE,
                      EVT_AUTH_COMPLETE,
-                     EVT_ENCRYPT_CHANGE,
                      EVT_CMD_STATUS,
                      0 };
 
@@ -516,14 +454,13 @@ static gboolean hci_setup(struct pairing_context *ctx) {
     return TRUE;
 }
 
-static struct pairing_context *create_context(const char *bda, uint16_t clock, gboolean encrypt,
+static struct pairing_context *create_context(const char *bda, uint16_t clock,
                                       btpair_cb cb, gpointer user_data, GError **err) {
     struct pairing_context *ctx;
 
     ctx = g_new0(struct pairing_context, 1);
     ctx->state     = STATE_DISCONNECTED;
     ctx->clock     = clock;
-    ctx->encrypt   = encrypt;
     ctx->func      = cb;
     ctx->user_data = user_data;
     if (bda)
@@ -566,6 +503,73 @@ static gboolean pair_start(struct pairing_context *ctx, GError **err) {
     return TRUE;
 }
 
+void remove_link_key(const char *bda) {
+    GIOChannel *io;
+    FILE *f, *tf = NULL;
+    int t;
+    bdaddr_t la;
+    char addr[18], path[PATH_MAX], tmp[PATH_MAX], *line;
+
+    hci_delete_link_key(bda);
+
+    if (hci_devba(HCI_DEV_ID, &la) < 0) {
+        error("hci_devba: %s", strerror(errno));
+        return;
+    }
+
+    ba2str(&la, addr);
+    snprintf(path, sizeof(path), "/var/lib/bluetooth/%s/linkkeys", addr);
+
+    f = fopen(path, "r");
+    if (f == NULL)
+        return;
+
+    if (flock(fileno(f), LOCK_SH) < 0) {
+        error("Unable to lock %s: %s", path, strerror(errno));
+        goto close;
+    }
+
+    umask(S_IWGRP | S_IWOTH);
+
+    snprintf(tmp, sizeof(tmp), "/var/lib/bluetooth/%s/.btpair_lk_tmp", addr);
+    t = open(tmp, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR);
+    if (t < 0) {
+        error("Unable to open %s for writing: %s", tmp, strerror(errno));
+        goto unlock;
+    }
+
+    tf = fdopen(t, "w");
+    if (tf == NULL) {
+        error("fdopen failed: %s", strerror(errno));
+        close(t);
+        unlink(tmp);
+        goto unlock;
+    }
+    
+    io = g_io_channel_unix_new(fileno(f));
+
+    while (g_io_channel_read_line(io, &line, NULL, NULL, NULL) == G_IO_STATUS_NORMAL) {
+        if (strncasecmp(line, bda, 17))
+            fputs(line, tf);
+        g_free(line);
+    }
+
+    g_io_channel_unref(io);
+
+unlock:
+    flock(fileno(f), LOCK_UN);
+
+close:
+    fclose(f);
+
+    if (tf) {
+        fclose(tf);
+        if (rename(tmp, path) < 0)
+            error("rename(%s, %s) failed: %s", tmp, path, strerror(errno));
+    }
+}
+
+
 void pair_cancel(struct pairing_context *ctx) {
     GError *err = NULL;
 
@@ -576,17 +580,20 @@ void pair_cancel(struct pairing_context *ctx) {
     pair_finalize(ctx, err);
 }
 
-struct pairing_context *pair_bda(const char *bda, uint16_t clock, gboolean encrypt,
-                                 btpair_cb cb, gpointer user_data, GError **err) {
+struct pairing_context *pair_bda(const char *bda, uint16_t clock,
+                                 btpair_cb cb, gpointer user_data,
+                                 GError **err) {
 
     int handle = -1;
     struct pairing_context *ctx;
 
-    ctx = create_context(bda, clock, encrypt, cb, user_data, err);
+    ctx = create_context(bda, clock, cb, user_data, err);
 
     if (!ctx)
         return NULL;
 
+    remove_link_key(bda);
+
     get_bda_handle(&ctx->bdaddr, &handle);
     if (handle >= 0) {
         ctx->handle = (uint16_t)handle;
@@ -601,12 +608,12 @@ struct pairing_context *pair_bda(const char *bda, uint16_t clock, gboolean encry
     return ctx;
 }
 
-struct pairing_context *pair_handle(uint16_t handle, gboolean encrypt,
+struct pairing_context *pair_handle(uint16_t handle,
                                     btpair_cb cb, gpointer user_data,
                                     GError **err) {
     struct pairing_context *ctx;
 
-    ctx = create_context(NULL, 0, encrypt, cb, user_data, err);
+    ctx = create_context(NULL, 0, cb, user_data, err);
 
     if (!ctx)
         return NULL;
