@@ -30,13 +30,16 @@ typedef struct pairing_context PairingContext;
 
 typedef void (*btpair_cb)(GError *err, gpointer user_data);
 
-PairingContext *pair_bda(const char *bda, uint16_t clock, gboolean encrypt,
-                         btpair_cb cb, gpointer user_data, GError **err);
+PairingContext *pair_bda(const char *bda, uint16_t clock,
+                         btpair_cb cb, gpointer user_data,
+                         GError **err);
 
-PairingContext *pair_handle(uint16_t handle, gboolean encrypt,
-                            btpair_cb cb, gpointer user_data, GError **err);
+PairingContext *pair_handle(uint16_t handle,
+                            btpair_cb cb, gpointer user_data,
+                            GError **err);
 
 void pair_cancel(PairingContext *ctx);
 
+void remove_link_key(const char *bda);
 
 #endif /* _BTPAIR_ENGINE_H_ */
