@@ -52,7 +52,6 @@ static char *program_name;
 /** Command line config options */
 typedef struct {
     gboolean user_handle;
-    gboolean encrypt;
     gboolean use_syslog;
     gboolean use_dbus;
     uint16_t clock;
@@ -64,7 +63,6 @@ static struct option const long_options[] = {
     {"version",  no_argument,       0, 'V'},
     {"stdio",    no_argument,       0, 's'},
     {"handle",   no_argument,       0, 'H'},
-    {"encrypt",  no_argument,       0, 'e'},
     {"dbus",     no_argument,       0, 'd'},
     {"clock",    required_argument, 0, 'c'},
     {NULL, 0, NULL, 0}
@@ -83,18 +81,11 @@ static void usage(int status) {
     printf("-DEBUG ");
 #endif
 
-#ifdef BUGGY_BT_HW
-    printf("+BUGGY_BT_HW ");
-#else
-    printf("-BUGGY_BT_HW ");
-#endif
-
     printf(
      "\nUsage: %s [OPTION] <BDA or handle>\n"
      "Options:\n"
      "-h, --help         Display this help and exit\n"
      "-V, --version      Output version information and exit\n"
-     "-e, --encrypt      Also request encryption for the connection\n"
      "-d, --dbus         Use D-BUS inteface\n"
      "-s, --stdio        Log to stdout instead of syslog\n"
      "-H, --handle       Use existing connection handle\n"
@@ -123,7 +114,6 @@ static int decode_switches(int argc, char *argv[], CmdOpt *cfg)
                     "h"   /* help    */
                     "V"   /* version */
                     "s"   /* stdio   */
-                    "e"   /* encrypt */
                     "c:"  /* clock   */
                     "d"   /* dbus   */
                     "H"   /* handle  */
@@ -140,10 +130,6 @@ static int decode_switches(int argc, char *argv[], CmdOpt *cfg)
                 cfg->use_syslog = FALSE;
                 break;
 
-            case 'e':
-                cfg->encrypt = TRUE;
-                break;
-
             case 'd':
                 cfg->use_dbus = TRUE;
                 break;
@@ -210,10 +196,10 @@ int main(int argc, char *argv[]) {
         if (cfg.user_handle) {
             int handle;
             handle = strtol(argv[i], NULL, 0);
-            ctx = pair_handle(handle, cfg.encrypt, (btpair_cb)cli_cb, &ret, &err);
+            ctx = pair_handle(handle, (btpair_cb)cli_cb, &ret, &err);
         }
         else
-            ctx = pair_bda(argv[i], cfg.clock, cfg.encrypt, (btpair_cb)cli_cb, &ret, &err);
+            ctx = pair_bda(argv[i], cfg.clock, (btpair_cb)cli_cb, &ret, &err);
 
         if (err) {
             error("btpair failed: %s", err->message);
