@@ -28,6 +28,8 @@
 
 #include <dbus/dbus.h>
 
+#include <bluetooth/bluetooth.h>
+
 #include "log.h"
 #include "dbus.h"
 #include "dbus-helper.h"
@@ -69,50 +71,6 @@ static gchar *read_pin(void) {
     return g_strdup(pin);
 }
 
-static DBusHandlerResult service_exit_filter(DBusConnection *connection,
-                                             DBusMessage *message,
-                                             void *user_data) {
-    char *service; 
-
-    if (dbus_message_is_signal(message,
-                               DBUS_INTERFACE_ORG_FREEDESKTOP_DBUS,
-                               "ServiceOwnerChanged")) {
-        char *old, *new;
-
-        if (!get_dbus_args(message,
-                    DBUS_TYPE_STRING, &service,
-                    DBUS_TYPE_STRING, &old,
-                    DBUS_TYPE_STRING, &new,
-                    DBUS_TYPE_INVALID)) {
-            error("Invalid arguments for ServiceOwnerChanged signal");
-            return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-        }
-
-        dbus_free(new);
-
-        /* Service was created */
-        if (old[0] == '\0') {
-            dbus_free(old);
-            return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-        }
-
-        dbus_free(old);
-    }
-    else
-        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-
-
-    if (!btpin_stack_unregister(service, NULL, NULL)) {
-        dbus_free(service);
-        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
-    }
-    else {
-        debug("Service \"%s\" exited without first calling \"unregister\"",
-                service);
-        dbus_free(service);
-        return DBUS_HANDLER_RESULT_HANDLED;
-    }
-}
 static gboolean send_pin_reply(DBusConnection *connection,
                                DBusMessage    *message,
                                const gchar    *pin) {
@@ -121,7 +79,7 @@ static gboolean send_pin_reply(DBusConnection *connection,
     if (pin != NULL) {
         debug("Replying with PIN: %s", pin);
         append_dbus_args(reply,
-                         DBUS_TYPE_STRING, pin,
+                         DBUS_TYPE_STRING, &pin,
                          DBUS_TYPE_INVALID);
     }
     else
@@ -135,21 +93,21 @@ static void notify_cb(DBusPendingCall *pending,
     DBusMessage *reply;
     gchar *pin = NULL;
 
-    reply = dbus_pending_call_get_reply(pending);
+    reply = dbus_pending_call_steal_reply(pending);
     if (reply == NULL)
         error("dbus_pending_call_get_reply returned NULL!");
     if (dbus_message_get_type(reply) == DBUS_MESSAGE_TYPE_ERROR)
         error("Got error reply: %s", dbus_message_get_error_name(reply));
     else {
-        if (!get_dbus_args(reply,
-                           DBUS_TYPE_STRING, &pin,
-                           DBUS_TYPE_INVALID))
+        if (!dbus_message_get_args(reply, NULL,
+                                   DBUS_TYPE_STRING, &pin,
+                                   DBUS_TYPE_INVALID))
             error("Invalid arguments in PIN reply");
         else
             debug("Got reply from handler: %s", pin);
     }
 
-    if (reply != NULL)
+    if (reply)
         dbus_message_unref(reply);
 
     connection = get_dbus_connection();
@@ -164,36 +122,28 @@ static void notify_cb(DBusPendingCall *pending,
 static DBusHandlerResult bluez_req_handler(DBusConnection     *connection,
                                            DBusMessage        *message,
                                            void               *user_data) {
-    gchar *bdastr = NULL, *service = NULL, *path = NULL;
-    unsigned char *bda = NULL;
-    dbus_bool_t out;
-    guint bdalen;
+    gchar *service = NULL, *path = NULL;
+    const char *dev_path, *address;
+    DBusPendingCall *pending;
+    DBusMessage *newmsg;
+    dbus_bool_t out = TRUE;
+    bdaddr_t ba;
+    char *ptr = (char *)&ba;
 
-    if (!get_dbus_args (message,
-                        DBUS_TYPE_BOOLEAN, &out,
-                        DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &bda, &bdalen,
-                        DBUS_TYPE_INVALID)) {
-        error("PinRequest with invalid arguments received");
-        if (!send_invalid_args(connection, message))
-            error("Unable to send invalid_args error");
-        return DBUS_HANDLER_RESULT_HANDLED;
+    if (!dbus_message_is_method_call(message, "org.bluez.PasskeyAgent", "Request"))
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+
+    if (!dbus_message_get_args(message, NULL,
+                               DBUS_TYPE_STRING, &dev_path,
+                               DBUS_TYPE_STRING, &address,
+                               DBUS_TYPE_INVALID)) {
+        error("Unexpected arguments to Security.Request");
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
     }
 
-    if (bdalen != 6) {
-        error("PinRequest with invalid BDA length: %u", bdalen);
-        if (!send_invalid_args(connection, message))
-            error("Unable to send invalid_args error");
-        dbus_free(bda);
-        return DBUS_HANDLER_RESULT_HANDLED;
-    }
+    debug("Request(%s, %s)", dev_path, address);
 
-    bdastr = g_strdup_printf("%02X:%02X:%02X:%02X:%02X:%02X",
-                             bda[5], bda[4], bda[3],
-                             bda[2], bda[1], bda[0]);
-
-    debug("PinRequest(%s, %s)", out ? "out" : "in", bdastr);
-
-    if (!btpin_stack_get_handler(bdastr, &service, &path)) {
+    if (!btpin_stack_get_handler(address, &service, &path)) {
         /* Fallback */
         gchar *pin;
         debug("No PIN-handler matched. Using fallback");
@@ -201,40 +151,36 @@ static DBusHandlerResult bluez_req_handler(DBusConnection     *connection,
         if (!send_pin_reply(connection, message, pin))
             error("Sending PIN reply failed");
         g_free(pin);
-    }
-    else {
-        DBusPendingCall *pending;
-        DBusMessage *newmsg;
-
-        debug("Found matching handler at (%s, %s)", service, path);
-
-        newmsg = new_dbus_method_call(service,
-                                      path,
-                                      dbus_message_get_interface(message),
-                                      dbus_message_get_member(message));
-        append_dbus_args(newmsg,
-                         DBUS_TYPE_BOOLEAN, out,
-                         DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, bda, bdalen,
-                         DBUS_TYPE_INVALID);
-
-        if (!dbus_connection_send_with_reply(connection, newmsg,
-                                             &pending, TIMEOUT))
-            die("Out of memory during dbus_connection_send_with_reply");
-
-        dbus_connection_flush(connection);
-        debug("PIN request was sent to handler. Waiting for reply.");
-
-        if (!dbus_pending_call_set_notify(pending,
-                                          (DBusPendingCallNotifyFunction)notify_cb,
-                                          message,
-                                          (DBusFreeFunction)dbus_message_unref))
-            die("Out of memory during dbus_pending_call_set_notify");
-
-        dbus_message_ref(message);
+        return DBUS_HANDLER_RESULT_HANDLED;
     }
 
-    dbus_free(bda);
-    g_free(bdastr);
+    debug("Found matching handler at (%s, %s)", service, path);
+
+    str2ba(address, &ba);
+
+    newmsg = new_dbus_method_call(service, path, PINAGENT_INTERFACE,
+                                  "PinRequest");
+
+    append_dbus_args(newmsg,
+            DBUS_TYPE_BOOLEAN, &out,
+            DBUS_TYPE_ARRAY, DBUS_TYPE_BYTE, &ptr, sizeof(bdaddr_t),
+            DBUS_TYPE_INVALID);
+
+    if (!dbus_connection_send_with_reply(connection, newmsg,
+                &pending, TIMEOUT))
+        die("Out of memory during dbus_connection_send_with_reply");
+
+    debug("PIN request was sent to handler. Waiting for reply.");
+
+    if (!dbus_pending_call_set_notify(pending,
+                (DBusPendingCallNotifyFunction)notify_cb,
+                message,
+                (DBusFreeFunction)dbus_message_unref))
+        die("Out of memory during dbus_pending_call_set_notify");
+
+    dbus_message_ref(message);
+
+    dbus_message_unref(newmsg);
     g_free(service);
     g_free(path);
 
@@ -249,14 +195,14 @@ static DBusHandlerResult btpin_req_handler(DBusConnection     *connection,
     const char *service;
 
     if (dbus_message_has_signature(message, bda_sgn))
-        (void) get_dbus_args(message,
-                             DBUS_TYPE_STRING, &path,
-                             DBUS_TYPE_STRING, &bda,
-                             DBUS_TYPE_INVALID);
+        (void) dbus_message_get_args(message, NULL,
+                                     DBUS_TYPE_STRING, &path,
+                                     DBUS_TYPE_STRING, &bda,
+                                     DBUS_TYPE_INVALID);
     else if (dbus_message_has_signature(message, nobda_sgn)) {
-        (void) get_dbus_args(message,
-                             DBUS_TYPE_STRING, &path,
-                             DBUS_TYPE_INVALID);
+        (void) dbus_message_get_args(message, NULL,
+                                     DBUS_TYPE_STRING, &path,
+                                     DBUS_TYPE_INVALID);
         bda = NULL;
     }
     else {
@@ -287,10 +233,6 @@ static DBusHandlerResult btpin_req_handler(DBusConnection     *connection,
                 error("Sending D-BUS reply failed");
         }
 
-        dbus_free(path);
-        if (bda != NULL)
-            dbus_free(bda);
-
         return DBUS_HANDLER_RESULT_HANDLED;
     }
 
@@ -315,21 +257,44 @@ static DBusHandlerResult btpin_req_handler(DBusConnection     *connection,
                 error("Sending D-BUS reply failed");
         }
 
-
-        dbus_free(path);
-        if (bda != NULL)
-            dbus_free(bda);
-
         return DBUS_HANDLER_RESULT_HANDLED;
     }
                                     
-    dbus_free(path);
-    if (bda != NULL)
-        dbus_free(bda);
-
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
+static gboolean register_default_handler(DBusConnection *conn, const char *path) {
+    DBusMessage *msg, *reply;
+    DBusError err;
+
+    msg = dbus_message_new_method_call("org.bluez", "/org/bluez",
+                                       "org.bluez.Security",
+                                       "RegisterDefaultPasskeyAgent");
+    if (!msg)
+        return FALSE;
+
+    if (!dbus_message_append_args(msg, DBUS_TYPE_STRING, &path,
+                                  DBUS_TYPE_INVALID)) {
+        dbus_message_unref(msg);
+        return FALSE;
+    }
+
+    dbus_error_init(&err);
+    reply = dbus_connection_send_with_reply_and_block(conn, msg, -1, &err);
+
+    dbus_message_unref(msg);
+
+    if (dbus_error_is_set(&err)) {
+        error("%s", err.message);
+        dbus_error_free(&err);
+        return FALSE;
+    }
+
+    dbus_message_unref(reply);
+
+    return TRUE;
+}
+
 static DBusObjectPathVTable bluez_req_vtable = {
     .message_function    = bluez_req_handler,
     .unregister_function = NULL
@@ -344,9 +309,6 @@ static DBusObjectPathVTable btpin_req_vtable = {
 void init_dbus_handlers(DBusConnection *connection) {
     DBusError derr;
 
-    debug("My base service is %s",
-            dbus_bus_get_base_service(connection));
-
     dbus_error_init(&derr);
     dbus_connection_register_object_path(connection,
                                          BLUEZ_PIN_AGENT_PATH,
@@ -358,14 +320,9 @@ void init_dbus_handlers(DBusConnection *connection) {
         dbus_error_free(&derr);
     }
 
-    dbus_bus_acquire_service(connection,
-                             BTPIN_SERVICE,
-                             DBUS_SERVICE_FLAG_PROHIBIT_REPLACEMENT,
-                             &derr);
-    if (dbus_error_is_set(&derr)) {
-        error("Aquiring service %s failed: %s",
-                BTPIN_SERVICE, derr.message);
-        dbus_error_free(&derr);
+    if (!register_default_handler(connection, BLUEZ_PIN_AGENT_PATH)) {
+        error("Registering default passkey handler failed");
+        return;
     }
 
     dbus_connection_register_object_path(connection,
@@ -377,19 +334,6 @@ void init_dbus_handlers(DBusConnection *connection) {
                 BTPIN_PATH, derr.message);
         dbus_error_free(&derr);
     }
-
-    if (!dbus_connection_add_filter(connection,
-                                    service_exit_filter, NULL, NULL))
-        die("Out of memory during dbus_connection_add_filter");
-
-    dbus_bus_add_match(connection,
-            "interface=" DBUS_INTERFACE_ORG_FREEDESKTOP_DBUS
-            ",member=ServiceOwnerChanged",
-            &derr);
-    if (dbus_error_is_set(&derr)) {
-        error("dbus_add_match() failed: %s", derr.message);
-        dbus_error_free(&derr);
-    }
 }
 
 void destroy_dbus_handlers(DBusConnection *connection) {
