@@ -29,6 +29,8 @@
 #include <dbus/dbus.h>
 
 #include "log.h"
+#include "dbus.h"
+#include "dbus-helper.h"
 #include "btpin-stack.h"
 
 typedef struct {
@@ -148,6 +150,11 @@ static void stack_remove(PinHandler *handler) {
     stack = g_slist_delete_link(stack, match);
 }
 
+static void name_exited(const char *name, PinHandler *handler) {
+    stack_remove(handler);
+    debug("\"%s\" exited without first calling \"unregister\"", name);
+}
+
 gboolean btpin_stack_register(const gchar *service,
                               const gchar *path,
                               const gchar *bda) {
@@ -166,8 +173,11 @@ gboolean btpin_stack_register(const gchar *service,
     }
 
     handler = pin_handler_new(service, path, bda);
+
     stack_add(handler);
 
+    add_name_listener(get_dbus_connection(), service, (name_cb)name_exited, handler);
+
     stack_show();
 
     return TRUE;
@@ -176,31 +186,32 @@ gboolean btpin_stack_register(const gchar *service,
 gboolean btpin_stack_unregister(const gchar *service,
                                 const gchar *path,
                                 const gchar *bda) {
-    gint i = 0;
+    gint count = 0;
 
     g_assert(service != NULL);
 
-    do {
+    while (TRUE) {
         PinHandler *handler;
 
         handler = stack_find_handler(service, path, bda);
 
-        if (handler != NULL) {
-            stack_remove(handler);
-            i++;
-        }
-        else
+        if (!handler)
             break;
 
-    } while (TRUE);
+        stack_remove(handler);
 
-    if (i > 0) {
-        debug("Removed %i handlers", i);
-        stack_show();
-        return TRUE;
+        remove_name_listener(get_dbus_connection(), service, (name_cb)name_exited, handler);
+
+        count++;
     }
-    else
+
+    if (!count)
         return FALSE;
+
+    debug("Removed %i %s", count, count > 1 ? "handlers" : "handler");
+    stack_show();
+
+    return TRUE;
 }
 
 gboolean btpin_stack_get_handler(const gchar *bda,
