@@ -205,7 +205,7 @@ int main(int argc, char *argv[]) {
 
     event_loop = g_main_loop_new(NULL, FALSE);
 
-    if (!setup_dbus_connection(PINAGENT_SERVICE, init_dbus_handlers))
+    if (!setup_dbus_connection(BTPIN_SERVICE, init_dbus_handlers))
         die("D-BUS connection setup failed!");
 
     report("BT PIN %s started.", VERSION);
