@@ -31,59 +31,58 @@
 #include "log.h"
 #include "dbus.h"
 
-static DBusConnection *_dbus_connection = NULL;
+static DBusConnection *connection = NULL;
 
 DBusConnection *get_dbus_connection(void) {
-    return _dbus_connection;
+    if (!connection)
+        connection = dbus_bus_get(DBUS_BUS_SYSTEM, NULL);
+    return connection;
 }
 
-gboolean setup_dbus_connection(const char *service,
+gboolean setup_dbus_connection(const char *name,
                           void (*handler_init)(DBusConnection *connection)) {
     DBusError derr;
 
-    g_assert(_dbus_connection == NULL);
+    g_assert(connection == NULL);
 
     dbus_error_init(&derr);
-    _dbus_connection = dbus_bus_get(DBUS_BUS_SYSTEM, &derr);
-    if (_dbus_connection == NULL) {
+    connection = dbus_bus_get(DBUS_BUS_SYSTEM, &derr);
+    if (connection == NULL) {
         error("System DBus connection failed: %s", derr.message);
         dbus_error_free(&derr);
         return FALSE;
     }
 
-    dbus_connection_setup_with_g_main(_dbus_connection, NULL);
+    dbus_connection_setup_with_g_main(connection, NULL);
 
-    if (service) {
+    if (name) {
         int ret;
 
-        ret = dbus_bus_acquire_service(_dbus_connection,
-                                       service,
-                                       DBUS_SERVICE_FLAG_PROHIBIT_REPLACEMENT,
-                                       &derr);
+        ret = dbus_bus_request_name(connection, name, 0, &derr);
         if (dbus_error_is_set(&derr)) {
-            error("Could not aquire DBUS service '%s'", service);
+            error("Could not aquire DBUS service '%s'", name);
             debug("%s: %s", derr.name, derr.message);
             dbus_error_free(&derr);
             return FALSE;
         }
 
-        if (ret != DBUS_SERVICE_REPLY_PRIMARY_OWNER) {
+        if (ret != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER) {
             error("Could not become primary owner of service '%s' (ret: %d)",
-                   service, ret);
+                   name, ret);
             return FALSE;
         }
     }
 
     if (handler_init)
-        handler_init(_dbus_connection);
+        handler_init(connection);
 
     return TRUE;
 }
 
 void close_dbus_connection(void) {
-    g_assert(_dbus_connection != NULL);
-    dbus_connection_unref(_dbus_connection);
-    _dbus_connection = NULL;
-    dbus_shutdown();
+    g_assert(connection != NULL);
+    dbus_connection_close(connection);
+    dbus_connection_unref(connection);
+    connection = NULL;
 }
 
