@@ -29,7 +29,7 @@
 /** Connect to the system D-Bus
  * @returns TRUE on success, FALSE on failure
  */
-gboolean setup_dbus_connection(const char *service,
+gboolean setup_dbus_connection(const char *name,
                           void (*handler_init)(DBusConnection *connection));
 
 /** Disconnect from the system D-Bus */
